/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.tycho.p2.publisher;

import java.util.ArrayList;
import java.util.Collection;

import org.sonatype.aether.graph.DependencyNode;
import org.sonatype.tycho.p2.publisher.model.Advice;

public class CompositePublisherLifecycleParticipant
    implements PublisherLifecycleParticipant
{

    private final PublisherLifecycleParticipant[] participants;

    public CompositePublisherLifecycleParticipant( final PublisherLifecycleParticipant... participants )
    {
        this.participants = participants;
    }

    public DependencyNode processDependencyTree( final DependencyNode root )
    {
        DependencyNode processed = root;
        for ( final PublisherLifecycleParticipant participant : participants )
        {
            processed = participant.processDependencyTree( processed );
        }
        return processed;
    }

    public Collection<Advice> getAdvices( final DependencyNode root )
    {
        final Collection<Advice> advices = new ArrayList<Advice>();
        for ( final PublisherLifecycleParticipant participant : participants )
        {
            advices.addAll( participant.getAdvices( root ) );
        }
        return advices;
    }

    public DependencyNode cleanupDependencyTree( final DependencyNode root )
    {
        DependencyNode processed = root;
        for ( final PublisherLifecycleParticipant participant : participants )
        {
            processed = participant.cleanupDependencyTree( processed );
        }
        return processed;
    }

    public static PublisherLifecycleParticipant composite( final PublisherLifecycleParticipant... participants )
    {
        return new CompositePublisherLifecycleParticipant( participants );
    }

}
