/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.tycho.p2.publisher;

import java.util.Collection;

import org.sonatype.aether.graph.DependencyNode;
import org.sonatype.tycho.p2.publisher.model.Advice;

public interface PublisherLifecycleParticipant
{
    /**
     * Hook for processing (adding / removing) nodes from dependency tree prior getting advices.
     * 
     * @param root dependency tree (root node)
     * @return new dependency tree (root node)
     */
    DependencyNode processDependencyTree( DependencyNode root );

    /**
     * Get advices contributed by this participant (eventually extracted from dependency tree);
     * 
     * @param root dependency tree (root node)
     * @return advices
     */
    Collection<Advice> getAdvices( DependencyNode root );

    /**
     * Hook for processing (adding / removing) nodes from dependency tree after getting advices.
     * 
     * @param root dependency tree (root node)
     * @return new dependency tree (root node)
     */
    DependencyNode cleanupDependencyTree( DependencyNode root );
}
