/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.tycho.p2.publisher.impl;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.codehaus.plexus.util.StringUtils;
import org.sonatype.aether.artifact.Artifact;

public class Utils
{

    static final Pattern FUZZY_VERSION =
        Pattern.compile( "(\\d+)(\\.(\\d+)(\\.(\\d+))?)?([^a-zA-Z0-9](.*))?", Pattern.DOTALL );

    public static String toOsgiVersion( final String version, final String timestamp )
    {
        final StringBuffer result = new StringBuffer();
        final Matcher m = FUZZY_VERSION.matcher( version );
        if ( m.matches() )
        {
            final String major = m.group( 1 );
            final String minor = m.group( 3 );
            final String micro = m.group( 5 );
            final String qualifier = m.group( 7 );

            if ( major != null )
            {
                result.append( major );
                if ( minor != null )
                {
                    result.append( "." );
                    result.append( minor );
                    if ( micro != null )
                    {
                        result.append( "." );
                        result.append( micro );
                        if ( qualifier != null )
                        {
                            result.append( "." );
                            cleanupModifier( result, qualifier );
                        }
                    }
                    else if ( qualifier != null )
                    {
                        result.append( ".0." );
                        cleanupModifier( result, qualifier );
                    }
                    else
                    {
                        result.append( ".0" );
                    }
                }
                else if ( qualifier != null )
                {
                    result.append( ".0.0." );
                    cleanupModifier( result, qualifier );
                }
                else
                {
                    result.append( ".0.0" );
                }
            }
        }
        else
        {
            result.append( "0.0.0." );
            cleanupModifier( result, version );
        }
        return result.toString().replace( "SNAPSHOT", timestamp );
    }

    private static void cleanupModifier( final StringBuffer result, final String modifier )
    {
        for ( int i = 0; i < modifier.length(); i++ )
        {
            final char c = modifier.charAt( i );
            if ( ( c >= '0' && c <= '9' ) || ( c >= 'a' && c <= 'z' ) || ( c >= 'A' && c <= 'Z' ) || c == '_'
                || c == '-' )
                result.append( c );
            else
                result.append( '_' );
        }
    }

    public static String toBuildVersion( final String version, final String timestamp )
    {
        String buildVersion = toOsgiVersion( version, timestamp );
        if ( !buildVersion.endsWith( timestamp ) )
        {
            if ( buildVersion.split( "\\." ).length == 4 )
            {
                buildVersion += "-" + timestamp;
            }
            else
            {
                buildVersion += "." + timestamp;
            }
        }
        return buildVersion;
    }

    public static String pathOf( final Artifact artifact )
    {
        final StringBuilder path = new StringBuilder( 128 );

        path.append( artifact.getGroupId().replace( '.', '/' ) ).append( '/' );

        path.append( artifact.getArtifactId() ).append( '/' );

        path.append( artifact.getBaseVersion() ).append( '/' );

        path.append( artifact.getArtifactId() ).append( '-' ).append( artifact.getVersion() );

        if ( artifact.getClassifier().length() > 0 )
        {
            path.append( '-' ).append( artifact.getClassifier() );
        }

        path.append( '.' ).append( artifact.getExtension() );

        return path.toString();
    }

    public static String fullArtifactId( final Artifact artifact )
    {
        return conflictArtifactId( artifact ) + ":" + artifact.getBaseVersion();
    }

    public static String conflictArtifactId( final Artifact artifact )
    {
        final StringBuilder sb = new StringBuilder( 128 );
        sb.append( artifact.getGroupId() );
        sb.append( ":" );
        sb.append( artifact.getArtifactId() );
        sb.append( ":" );
        sb.append( artifact.getExtension() );
        if ( StringUtils.isNotEmpty( artifact.getClassifier() ) )
        {
            sb.append( ":" );
            sb.append( artifact.getClassifier() );
        }
        return sb.toString();
    }

    public static String p2Id( final String id, final Artifact artifact )
    {
        final StringBuilder sb = new StringBuilder( 128 );

        sb.append( id );
        sb.append( "/" );
        sb.append( conflictArtifactId( artifact ) );

        return sb.toString();
    }

}
